%% Header
% Purpose:  Calculation and graphical representation of SIMC 
%           tuning performance
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% Org.:     Department of Chemical Engineering, NTNU, Norway
% About:    ctrl = [Kc I D]

%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

runTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10',...
             'case11', 'case12', 'case13'} ;
MsRes = 0.025 ;                          % Ms resolution for plot
parm.simTime = 100 ;

opt = optimset('Display','off') ;   % fzero options

    fprintf('Generating SIMC PI curve\n')
    fprintf('Main Directory: %s\n', main)

for modelId = 13:13
    calcTime = cputime ;
    %% Extracting data fra PID optimization
    if modelId < 10
        data = load([main,'/results/pid/chriss/','POpid_',...
                num2str(modelName{modelId}),'.mat']) ;
        IAE0_y = data.res.POpid.weighting.IAE0_y ;
        IAE0_d = data.res.POpid.weighting.IAE0_d ;
    elseif modelId > 9
        data = load([main,'/results/pi/','POpi_',...
                num2str(modelName{modelId}),'.mat']) ;
        IAE0_y = data.res.IAEweights(1) ;
        IAE0_d = data.res.IAEweights(2) ;
    end
    
    fprintf('IAE weights loaded.\n')
    fprintf('IAE0_y = %0.2f\n', IAE0_y)
    fprintf('IAE0_d = %0.2f\n', IAE0_d)

    parm.Jweights   = [0.5 0.5] ;         % Cost function weights
    parm.IAEweights = [IAE0_y IAE0_d]' ;            % IAE weights

    G = model(modelId) ;                                  % Model
    H = halfrule(G) ;                 % Reduced first order model
    res.G = G ;                                % Saving to struct
    res.H = H ;                                % Saving to struct

    tau_start = -1/4*G.theta ;
    tau_end = 4*G.theta ;
    tau_res = (tau_end - tau_start) / 80 ;     % tau_c resolution

    % Generating optimal controllers
    i = 1 ;                                   % Iteration counter
    tau_span = tau_start:tau_res:tau_end ;
    Ms_span = zeros(length(tau_span),1) ;         % Values for Ms
    Mt_span = zeros(length(tau_span),1) ;       % Init. Mt vector
    sTime   = zeros(length(tau_span),1) ;% Init. time consumption
    Jtune   = zeros(length(tau_span),1) ;     % Init. cost vector
    flags   = zeros(length(tau_span),1) ;% Init. exit flags vector
    IAEtune = zeros(length(tau_span),2) ;      % Init. IAE vector
    minTune = zeros(length(tau_span),2) ;   % Init. tuning vector
    initial = zeros(length(tau_span),2) ;% Initial values for each iteration
    ctrls   = zpk(zeros(length(tau_span),1)) ; % Transfer functions for controllers
    ctrlRoot{length(tau_span)} = [] ; % Roots of produced controllers

    fprintf('Case: %g\n',modelId)
    fprintf('Controller: SIMC PI\n')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(tau_span))
    fprintf('J \t K \t I \t Ms \t tau_c \n')
    fprintf('------------------------------------------------\n')

    if modelId > 9
        H = G;
    end

    for tau_c = tau_span
        C            = simcController(tau_c, H) ;
        ctrl         = [C.Kc C.tau_i] ;
        J            = simcCost(C, parm) ;
        Ms_span(i)   = MsSimc(C, parm) ;
        Mt_span(i)   = MtSimc(C, parm) ;
        minTune(i,:) = ctrl ;        %  Storing controller tuning
        ctrls(i)     = C.tf ;             % Storing controller TF
        ctrlRoot{i}  = zpkdata(C.tf) ;         % Extracting roots
        IAEtune(i,:) = iaeSimc(C, parm) ;    % Saving IAE weights
        Jtune(i)     = J ;          % Storing cost function value
        fprintf('%.2f \t %.2f \t %.2f \t %.2f \t %.3f \t%.3f',...
            J,ctrl,Ms_span(i), tau_c)
        fprintf('\n')
        i = i + 1 ;                        % Incrementing counter
    end

    calcTime = cputime - calcTime ;    % End of total comp. time

    %% Plotting routine
    figure(modelId)             % Figure number from model number
    clf                                            % Clear figure
    h(1) = plot(Ms_span,Jtune);     % Plotting Ms and cost values
    axis([0 3 0 6])                             % Adjusting axis'
    xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
    ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
    titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
                 'Case 5', 'Case 6', 'Case 7', 'Case 8', ...
                 'Case 9', 'Case 10', 'Case 11', 'Case 12',...
                 'Case 13'} ;
    title(titleName{modelId},'interpreter','latex','FontSize',12)

    %% Storing results
    res.case = num2str(modelName{modelId});
    res.ctrlType        = 'SIMC PI' ;
    res.IAEweights      = parm.IAEweights ;
    res.POsimcPI.tau_c  = tau_span ;
    res.POsimcPI.Ms     = Ms_span ;
    res.POsimcPI.Mt     = Mt_span ;
    res.POsimcPI.tuning = minTune ;
    res.POsimcPI.Jtune  = Jtune ;
    res.POsimcPI.ctrls  = ctrls ; 
    res.POsimcPI.cTime  = calcTime ;
    res.POsimcPI.Jy     = IAEtune(:,1) ;
    res.POsimcPI.Jd     = IAEtune(:,2) ;
    res.POsimcPI.ctrls  = ctrls ; 
    res.POsimcPI.roots  = ctrlRoot ;

    save([main,'/results/pi/simc/','POsimcPI_',...
          num2str(modelName{modelId}),'.mat'],'res')   % Saving struct
    saveas(h(1),[main,'/figures/pi/simc/','POsimcPIPIcurve_',...
          num2str(modelName{modelId}),'.fig'])      % Saving figures

    fprintf(...
        'Calculation finished after %0.2f minutes ',calcTime/60) 
    fprintf(...
        '(%0.2f seconds).\n\n', calcTime)

end                                             % Ending for-loop
runTime = cputime - runTime;
fprintf('Total computation time: %2.2f minutes', runTime/60)
fprintf(' (%2.2f seconds). \n\n', runTime)
diary off
restoredefaultpath