%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum ctrlr
calcWeights = 1 ;
runTime = cputime ;
warning = off;

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10', ...
             'case11', 'case12', 'case13'} ;
ctrlType  = {'Chriss controller', 'Parallel', ...
             'Alternative parallel', 'Cascade'} ;

for modelId = 13:13 ;                                  % Model ID
cTime = cputime ;              % Start total computation time
%% Start diary
    diary([main,'/diary/pi/','diary_POpi_',...
          num2str(modelName{modelId}),'.txt']) ;
cNum    = 4 ;                                 % Controller option
sNum    = 2 ;                                     % Solver option
ctrlr   = 'pi' ;
solvers = {'active-set', 'SQP'} ;   % Solvers
sFlags  = {5,1} ;                % Solver flag limits for success
solver  = solvers{sNum} ;                       % Choosing solver
sFlag   = sFlags{sNum}  ;                   % Choosing flag limit

simTime = 100 ;                                 % Simulation time
Ms_s    = 1.59 ;
MsRes   = 0.025 ;                         % Ms PO plot resolution
G       = model(modelId) ;  % Generating system transfer function

% Optimizer options
parm.simTime    = simTime ;
parm.Ms_s       = Ms_s ;
parm.IAEweights = [1 1]' ;
init = initValues(modelId) ;             % Initial control matrix
lb = zeros(size(init,2),1) ;                 % Lower tuning bound

opt = optimset('algorithm',solver,'Display','off','TolCon',...
        1e-4,'PlotFcns',...
        {@optimplotx, @optimplotfval, @optimplotfunccount}); 
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('=== PARETO OPTIMAL PI ===\n')
    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Controller: %s\n', ctrlType{cNum})
    fprintf('Solver: %s\n', opt.Algorithm)

if calcWeights == 1
    init = initValues(modelId) ;
    parm.Jweights = [1 0] ;
    fprintf('\nFinding Optimal IAE Weights \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning\n')
    fprintf('J     \t Kc \t I \t Ms_s\tMs \t flag \t CPU time\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    ctrl = init(1,:) ;
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm) ;
    try
        tic ;
        [ctrl iae_y optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
    catch
        optFlag = 0 ;
    end
    if optFlag < sFlags{sNum}
        Jfun  = @(ID)noGradient([ctrl(1) ID], parm) ; % fsearch help function
        tic ;
        [ID iae_y optFlag] = fminsearch(Jfun, ctrl(2:end), opt) ;
        sTime = toc ;               % Ending CPU time measurement
        MsFun = @(Kc) parm.Ms_s - Ms([ctrl(1) ID]) ;   % fzero fcn
        Kc = fzero(MsFun, ctrl(1)) ;            % Finding Kc
        ctrl = [Kc ID] ;
        try
            tic ;
        [ctrl iae_y optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
        catch
            fprintf('%s failed. Fminsearch results yields.\n',...
                    upper(solvers{sNum}))
        end
    end
    Msc  = Ms(ctrl) ;  % Calc. Ms for confirmation
    res.iaey = ctrl ;
    fprintf('%0.2f\t%.2f\t%.2f\t%.3f\t%.3f%d\t%3.2f\n',...
            iae_y,ctrl,Ms_s,Msc,optFlag,sTime)

    % Optimizing for IAE0_d
    init          = initValues(modelId) ; % Extracting initial values
    ctrl          = init(2,:) ;           % IAE0_d starting point
    parm.Jweights = [0 1] ;
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm) ;
    try
        tic ;
        [ctrl iae_d optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
    catch
        Jfun  = @(ID)noGradient([ctrl(1) ID], parm) ; % fsearch help function
        tic ;
        [ID iae_d optFlag] = fminsearch(Jfun, ctrl(2:end), opt) ;
        sTime = toc ;               % Ending CPU time measurement
        MsFun = @(Kc) parm.Ms_s - Ms([Kc ID]) ;   % fzero fcn
        Kc = fzero(MsFun, ctrl(1)) ;            % Finding Kc
        ctrl = [Kc ID] ;
    end
    Msc  = Ms(ctrl) ;  % Calc. Ms for confirmation
    res.iaed = ctrl ;
    fprintf('%0.2f\t%.2f\t%.2f\t%.3f\t%.3f%d\t%3.2f\n',...
            iae_d,ctrl,Ms_s,Msc,optFlag,sTime)
else
    %% Extracting data fra PID optimization
    data = load([main,'/results/pid/chriss/','POpid_',...
           num2str(modelName{modelId}),'.mat']) ;
    iae_y = data.res.POpid.weighting.IAE0_y ;
    iae_d = data.res.POpid.weighting.IAE0_d ;

        fprintf('\nIAE weights loaded.\n')
        fprintf('iae_y = %0.2f\n', iae_y)
        fprintf('iae_d = %0.2f\n', iae_d)
end

%% Pareto optimization
% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [iae_y iae_d]' ;                % IAE weights
ctrl = init(3,:) ;      % Setting initial values for pareto curve

% Generating pareto curve
i = 1 ;                                       % Iteration counter
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;           % Init. IAE vector
minTune = zeros(length(Ms_span),2) ;        % Init. tuning vector
initial = zeros(length(Ms_span),2) ;    % Initial values at each iteration
ctrls   = zpk(zeros(length(Ms_span),1)) ; % Transfer functions for controllers
ctrlRoot{length(Ms_span)} = [] ; % Roots of produced controllers

    fprintf('\nGenerating the Pareto Optimal Curve\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('J\tK\tI\tMs_s\tMs\tflag\titer.\tCPU time[s]\ttimeleft[min]\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun   = @(ctrl)costFun(ctrl, parm) ;         % Updating cost
    cFun = @(ctrl)conFun(ctrl,parm);       % Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    try
        tic ;                     % Starting CPU time measurement
        [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                           [],cFun,opt);   % Optimizing from last 
                                                  % optimal point
        sTime(i) = toc ;            % Ending CPU time measurement
    catch
        optFlag = 0 ;
    end
    if optFlag < sFlag         % Gradient free opt upon exit flag
        fprintf('%s algorithm failed. Using fminsearch:\n', ...
                    opt.Algorithm)
        if i == 1
            ctrl = init(3,:) ;         % Resetting initial values
        else
            ctrl = minTune(i-1,:) ; % Resetting last working ctrl
        end
        Kc = ctrl(1) ;           % Setting Kc for noGradient
        ID = ctrl(2:end) ;        % Setting I for noGradient
        Jfun  = @(ID)noGradient(ID,parm) ;% fsearch help function
        tic ;                     % Starting CPU time measurement
        [ID J optFlag] = ...
                       fminsearch(Jfun,ID, searchOpt) ;
        sTime(i) = toc ;            % Ending CPU time measurement
        MsFun = @(Kc) Ms_s - Ms([Kc ID]) ;% fzero help function
        Kc = fzero(MsFun, Kc) ;% Finding controller gain
        ctrl = [Kc ID] ; % Generating controller vector
        Msc  = Ms(ctrl) ;    % Calculating Ms for confirmation
        if (Msc > Ms_s + 1e-4)||(Msc < Ms_s - 1e-4)
            fprintf('Ms value differs at control calculation. ')
            fprintf('Msc = %0.4f\n',Msc)
        end
    end
    C = controller(ctrl) ;          % Producing controller struct
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Mt_span(i)   = Mt(ctrl) ;% Calculating corresponding Mt-value
    ctrls(i)     = C.tf ;                 % Storing controller TF
    ctrlRoot{i}  = zpkdata(C.tf) ;             % Extracting roots
    IAEtune(i,:) = iae(C, parm) ;            % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]

    fprintf('%.2f\t%.2f\t%.2f\t%.3f\t%0.3f\t%d\t%d\t%3.2f\t%4.2f',...
            J,ctrl,Ms_s,Msc,optFlag,length(Ms_span)-i,...
            sTime(i),timeleft) ;
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

cTime = cputime - cTime ;  % End of total computation time

%% Ploting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(1) = plot(Ms_span,Jtune);         % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8', 'Case 9'...
            'Case 10', 'Case 11', 'Case 12', 'Case13'};
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results
res.case = num2str(modelName{modelId});
res.ctrlType     = ctrlType{cNum} ;
res.G            = G ;
res.IAEweights   = parm.IAEweights ;
res.solver       = solver ;
res.POpi.Ms      = Ms_span ;
res.POpi.Mt      = Mt_span ;
res.POpi.tuning  = minTune ;
res.POpi.Jtune   = Jtune ;
res.POpi.flags   = flags ;
res.POpi.ctrls   = ctrls ; 
res.POpi.initial = initial ;
res.POpi.cTime   = cTime ;
res.POpi.Jy      = IAEtune(:,1) ;
res.POpi.Jd      = IAEtune(:,2) ;
res.POpi.ctrls   = ctrls ; 
res.POpi.roots   = ctrlRoot ;

% Saving struct and plot
save([main,'/results/pi/','POpi_',...
      num2str(modelName{modelId}),'.mat'],'res')
saveas(h(1),[main,'/figures/pi/','POpiCurve_',...
      num2str(modelName{modelId}),'.fig'])

fprintf('Calculation finished after %0.2f minutes', cTime/60)
fprintf(' (%0.2f seconds). \n', cTime)

end                                             % Ending for-loop
runTime = cputime - runTime ;
fprintf('Total computation time: %2.2f minutes', runTime/60)
fprintf(' (%2.2f seconds). \n\n', runTime)
diary off
restoredefaultpath