%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

startTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9'} ;
modelId = 6 ;
solver = 'active-set' ;
cNum    = 1 ;                                 % Controller option
simTime = 100 ;                                 % Simulation time
MsRes   = 0.025 ;                         % Ms PO plot resolution
G       = model(modelId) ;  % Generating system transfer function
res.G   = G ;                   % Saving system transfer function

%% Start diary
diary([main,'/diary/pi/','diary_POpi_',...
       num2str(modelName{modelId}),'.txt'])

%% Parameters for finding IAE weights
parm.Ms_s = 1.59 ;                                  % Ms setpoint
parm.simTime = simTime ;               % Response simulation time
parm.IAEweights = [1 1]' ;   % Setting IAE cost function weigting

init = initValuesPI(modelId) ;           % Initial control matrix
lb = [0 0]' ;                                % Lower tuning bound


% Optimizer options
opt = optimset('algorithm',solver,'Display','off','TolCon',...
        1e-4,'PlotFcns',...
        {@optimplotx, @optimplotfval, @optimplotfunccount}); 
    % 'active-set', 'trust-region-reflective', 'interior-point',
    % 'interior-point-convex', 'levenberg-marquardt', 
    % 'trust-region-dogleg', 'lm-line-search', or 'sqp'.
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('PI CONTROLLER\n')
    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Solver: %s\n', opt.Algorithm)
    fprintf('Finding Optimal IAE Weights \n')
    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning  \t         \t \n')
    fprintf('J     \t Kc \t I \t Ms \t flag \t CPU time\n')
    fprintf('------------------------------------------------\n')

%% Optimizing for IAE0_y
parm.Jweights   = [1 0]  ;        % Setting cost function weights
ctrl = init(1,:) ;            % Setting initial values for IAE0_y
Jfun = @(ctrl)costFun(ctrl, parm) ;
cFun = @(ctrl)conFun(ctrl, parm);
tic ;                             % Starting CPU time measurement
[ctrl IAE0_y optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                        [],cFun,opt);
sTime = toc ;                       % Ending CPU time measurement
mp(1) = get(0,'CurrentFigure') ;  % Saving optimization exit plot
res.POpi.weighting.ctrl_y = ctrl ;     % Saving controller settings
res.POpi.weighting.IAE0_y = IAE0_y ;      % Saving IAE input weight
res.POpi.weighting.flag_y = optFlag ;            % Saving exit flag

    fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %d \t %0.2f \n',...
        IAE0_y,ctrl,parm.Ms_s,optFlag,sTime)


% Optimizing for IAE0_d
parm.Jweights = [0 1]  ;
ctrl = init(2,:) ;            % Setting initial values for IAE0_d
Jfun   = @(ctrl)costFun(ctrl, parm) ;
cFun = @(ctrl)conFun(ctrl,parm);
tic ;                             % Starting CPU time measurement
[ctrl IAE0_d optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                        [],cFun,opt);
sTime = toc ;                       % Ending CPU time measurement
mp(2) = get(0,'CurrentFigure') ;  % Saving optimization exit plot
res.POpi.weighting.ctrl_d = ctrl ;   % Saving controller settings
res.POpi.weighting.IAE0_d = IAE0_d ;% Saving IAE disturbance weight
res.POpi.weighting.flag_d = optFlag ;         % Saving exit flags

    fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %d \t %0.2f \n',...
        IAE0_d,ctrl,parm.Ms_s,optFlag,sTime)

%% Pareto optimization
% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights
ctrl = init(3,:) ;      % Setting initial values for pareto curve

% Generating pareto curve
i = 1 ;                                       % Iteration counter
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;           % Init. IAE vector
minTune = zeros(length(Ms_span),2) ;        % Init. tuning vector
initial = zeros(length(Ms_span),2) ;    % Initial values for each iteration
ctrls   = zpk(zeros(length(Ms_span),1)) ; % Transfer functions for controllers
ctrlRoot{length(Ms_span)} = [] ; % Roots of produced controllers

    fprintf('\n\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J \t minTune \t Ms \t flag \t iter. \t CPU time[s] \t timeleft[min] \n')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun   = @(ctrl)costFun(ctrl, parm) ;         % Updating cost
    cFun = @(ctrl)conFun(ctrl,parm);       % Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    tic ;                         % Starting CPU time measurement
    [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                       [],cFun,opt);       % Optimizing from last 
                                                  % optimal point
    sTime(i) = toc ;                % Ending CPU time measurement
    if optFlag < 1    % Gradient free optimization upon exit flag
        fprintf('%s algorithm failed. Using fminsearch:\n', ...
                    opt.Algorithm)
        if i == 1
            ctrl = init(3,:) ;         % Resetting initial values
        else
            ctrl = minTune(i-1,:) ; % Resetting last working ctrl
        end
        parm.Kc = ctrl(1) ;           % Setting Kc for noGradient
        parm.ID = ctrl(2:end) ;        % Setting I for noGradient
        Jfun  = @(ID)noGradient(ID,parm) ;% fsearch help function
        tic ;                     % Starting CPU time measurement
        [parm.ID J optFlag] = ...
                       fminsearch(Jfun,parm.ID, searchOpt) ;
        sTime(i) = toc ;            % Ending CPU time measurement
        MsFun = @(Kc) Ms_s - Ms([Kc parm.ID]) ;% fzero help function
        parm.Kc = fzero(MsFun,parm.Kc) ;% Finding controller gain
        ctrl = [parm.Kc parm.ID] ; % Generating controller vector
        MsCalc  = Ms(ctrl) ;    % Calculating Ms for confirmation
        if MsCalc ~= abs(Ms_s+1e-4)
            fprintf('\n !!! Ms value differs at control calculation. MsCalc = %0.4f\n\n',MsCalc)
        end
    end
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Mt_span(i) = Mt(ctrl) ;  % Calculating corresponding Mt-value
    C = controller(ctrl) ;          % Producing controller struct
    ctrls(i) = C.tf ;                     % Storing controller TF
    ctrlRoot{i} = zpkdata(C.tf) ;              % Extracting roots
    IAEtune(i,:) = iae(ctrl, parm) ;         % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]

    fprintf('%.2f \t %.2f \t %.2f \t %.3f \t %d \t %d \t %0.2f \t\t %0.2f',...
        J,ctrl,Ms_s,optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

calcTime = cputime - startTime ;  % End of total computation time

%% Ploting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(1) = plot(Ms_span,Jtune);         % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8', 'Case 9'};
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results

res.case = num2str(modelName{modelId});
res.POpi.Ms     = Ms_span ;
res.POpi.Mt     = Mt_span ;
res.POpi.tuning = minTune ;
res.POpi.Jtune  = Jtune ;
res.POpi.flags  = flags ;
res.POpi.ctrls  = ctrls ; 
res.POpi.initial = initial ;
res.POpi.calcTime = calcTime ;
res.POpi.Jy = IAEtune(:,1) ;
res.POpi.Jd = IAEtune(:,2) ;
res.POpi.ctrls = ctrls ; 
res.POpi.roots = ctrlRoot ;

save([main,'/results/pi/','resPOpi_',...
     num2str(modelName{modelId}),'.mat'],'res')   % Saving struct
saveas(h(1),[main,'/figures/pi/','POpiCurve_',...
       num2str(modelName{modelId}),'.fig'])      % Saving figures

fprintf('Calculation finished after %0.2f minutes (%0.2f seconds). \n', calcTime/60, calcTime)

diary off
restoredefaultpath