%% Header
% Purpose:  Calculation and graphical representation of SIMC 
%           tuning performance
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% Org.:     Department of Chemical Engineering, NTNU, Norway
% About:    ctrl = [Kc I D]

%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

startTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10', ...
             'case11', 'case12', 'case13'} ;
MsRes = 0.025 ;                          % Ms resolution for plot
parm.simTime = 100 ;

opt = optimset('Display','off') ;   % fzero options

    fprintf('Generating SIMC PID curve\n')
    fprintf('Main Directory: %s\n', main)

for modelId = 10:13
    %% Extracting data fra PID optimization
    data = load([main,'/results/pid/chriss/','POpid_',...
                num2str(modelName{modelId}),'.mat']) ;
    IAE0_y = data.res.POpid.weighting.IAE0_y ;
    IAE0_d = data.res.POpid.weighting.IAE0_d ;

    fprintf('IAE weights loaded.\n')
    fprintf('IAE0_y = %0.2f\n', IAE0_y)
    fprintf('IAE0_d = %0.2f\n', IAE0_d)

    parm.Jweights   = [0.5 0.5] ;         % Cost function weights
    parm.IAEweights = [IAE0_y IAE0_d]' ;            % IAE weights

    G = model(modelId) ;                                  % Model
    res.G = G ;                                % Saving to struct

    tau_start = -1/4*G.theta ;
    tau_end = 4*G.theta ;
    tau_res = (tau_end - tau_start) / 80 ;     % tau_c resolution

    % Generating optimal controllers
    i = 1 ;                                   % Iteration counter
    tau_span = tau_start:tau_res:tau_end ;
    Ms_span = zeros(length(tau_span),1) ;         % Values for Ms
    Mt_span = zeros(length(tau_span),1) ;       % Init. Mt vector
    sTime   = zeros(length(tau_span),1) ;% Init. time consumption
    Jtune   = zeros(length(tau_span),1) ;     % Init. cost vector
    flags   = zeros(length(tau_span),1) ;% Init. exit flags vector
    IAEtune = zeros(length(tau_span),2) ;      % Init. IAE vector
    minTune = zeros(length(tau_span),3) ;   % Init. tuning vector
    initial = zeros(length(tau_span),3) ;% Initial values for each iteration
    ctrls   = zpk(zeros(length(tau_span),1)) ; % Transfer functions for controllers
    ctrlRoot{length(tau_span)} = [] ; % Roots of produced controllers

    fprintf('Case: %g\n',modelId)
    fprintf('Controller: SIMC PID\n')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(tau_span))
    fprintf('J \t K \t I \t D \t Ms \t tau_c \n')
    fprintf('------------------------------------------------\n')

    for tau_c = tau_span
        C            = simcController(tau_c, G) ;
        ctrl         = [C.Kc C.tau_i C.tau_d] ;
        J            = simcCost(C, parm) ;
        Ms_span(i)   = MsSimc(C, parm) ;
        Mt_span(i)   = MtSimc(C, parm) ;
        minTune(i,:) = ctrl ;        %  Storing controller tuning
        ctrls(i)     = C.tf ;             % Storing controller TF
        ctrlRoot{i}  = zpkdata(C.tf) ;         % Extracting roots
        IAEtune(i,:) = iaeSimc(C, parm) ;    % Saving IAE weights
        Jtune(i)     = J ;          % Storing cost function value
        fprintf('%.2f \t %.2f \t %.2f \t %.2f \t %.3f \t%.3f',...
            J,ctrl,Ms_span(i), tau_c)
        fprintf('\n')
        i = i + 1 ;                        % Incrementing counter
    end

    calcTime = cputime - startTime ;    % End of total comp. time

    %% Plotting routine
    figure(modelId)             % Figure number from model number
    clf                                            % Clear figure
    h(1) = plot(Ms_span,Jtune);     % Plotting Ms and cost values
    axis([0 3 0 6])                             % Adjusting axis'
    xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
    ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
    titleName = {'Case 1', 'Case 2',  'Case 3',  'Case 4', ...
                 'Case 5', 'Case 6',  'Case 7',  'Case 8', ...
                 'Case 9', 'Case 10', 'Case 11', 'Case 12', ...
                 'Case 13'} ;
    title(titleName{modelId},'interpreter','latex','FontSize',12)

    %% Storing results
    res.case = num2str(modelName{modelId});
    res.ctrlType         = 'SIMC PID' ;
    res.IAEweights       = parm.IAEweights ;
    res.POsimcPID.tau_c  = tau_span ;
    res.POsimcPID.Ms     = Ms_span ;
    res.POsimcPID.Mt     = Mt_span ;
    res.POsimcPID.tuning = minTune ;
    res.POsimcPID.Jtune  = Jtune ;
    res.POsimcPID.ctrls  = ctrls ; 
    res.POsimcPID.cTime  = calcTime ;
    res.POsimcPID.Jy     = IAEtune(:,1) ;
    res.POsimcPID.Jd     = IAEtune(:,2) ;
    res.POsimcPID.ctrls  = ctrls ; 
    res.POsimcPID.roots  = ctrlRoot ;

    save([main,'/results/pid/simc/','POsimcPID_',...
          num2str(modelName{modelId}),'.mat'],'res')   % Saving struct
    saveas(h(1),[main,'/figures/pid/simc/','POsimcCurve_',...
          num2str(modelName{modelId}),'.fig'])      % Saving figures

    fprintf(...
        'Calculation finished after %0.2f minutes ',calcTime/60) 
    fprintf(...
        '(%0.2f seconds).\n\n', calcTime)

end                                             % Ending for-loop
diary off
restoredefaultpath