%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

matlabpool size ;
if ans == 1
    matlabpool
end

%% Globals
global G cNum

runTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
calcWeights = 1 ; % Calculate IAE weights? Y(1), N(0)
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10', ...
             'case11', 'case12', 'case13'} ;
ctrlType  = {'po', 'simc', 'smith'} ;
cType  = 1
ctrlrs = {'pid', 'pi'} ;
ctrlr  = 1 ;
cParm  = {'Chriss', 'Parallel', 'AltParallel', 'Cascade'} ;
cNum   = 1 ;

sNum_s  = 2 ; 
sNum    = 2 ;                                     % Solver option
solvers = {'active-set', 'sqp', 'fsearch'} ;   % Solvers
sFlags  = {5,2,1} ;              % Solver flag limits for success
solver  = solvers{sNum} ;                       % Choosing solver
sFlag   = sFlags{sNum}  ;                   % Choosing flag limit
MsRes   = 0.025 ;                         % Ms PO plot resolution

for modelId = 12:13 ;                                 % Model ID
startTime = cputime ;
G = model(modelId) ;

%% Start diary
diary([main, '/diary/', num2str(ctrlType{cType}), ...
      '/', num2str(ctrlrs{ctrlr}),'/', ...
      '/', num2str(cParm{cNum}),'/', ...
      num2str(modelName{modelId}),'.txt'])

%% Parameters for finding IAE weights
parm.Ms_s = 1.59 ;                                  % Ms setpoint
parm.simTime = simTime ;               % Response simulation time
init = smithInit(modelId) ;   % Initial control matrix
lb = zeros(size(init,2),1) ;                 % Lower tuning bound

% Optimizer options
opt = optimoptions('fmincon','algorithm',solver,'UseParallel',...
                   'always','Display','off','TolCon',...
                1e-4,'PlotFcns',{@optimplotx, ...
                @optimplotfval, @optimplotfunccount}); 
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('%s CONTROLLER \n', ctrlr)
    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Controller: %s\n', upper(ctrlType{cType})) ;
    fprintf('Structure: %s\n', upper(ctrlrs{ctrlr}) ;
    fprintf('Parametrization: %s\n', cParm{cNum})
    fprintf('Solver: %s\n', solvers{sNum})
    
if calcWeights
    fprintf('\nFinding Optimal IAE Weights \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    if ctrlr == 1
        fprintf('J\tKc\tI\tD\tMs_s\tMs\tflag\tCPU time\n')
    else
        fprintf('J\tKc\tI\tMs\tflag\tCPU time\n')
    end
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    %% Optimizing for IAE0_y
    parm.IAEweights = [1 1]' ;     % Setting IAE costFun weigting
    parm.Jweights   = [1 0]  ;          % Setting costFun weights
    ctrl = init(1,:) ;
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm) ;
    try
        tic ;
        [ctrl iae_y optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
    catch
        optFlag = 0 ;
    end
    if optFlag < sFlags{sNum}
        Jfun  = @(ID)noGradient([ctrl(1) ID], parm) ; % fsearch help function
        tic ;
        [ID iae_y optFlag] = fminsearch(Jfun, ctrl(2:end), opt) ;
        sTime = toc ;               % Ending CPU time measurement
        MsFun = @(Kc) parm.Ms_s - Ms([ctrl(1) ID]) ;   % fzero fcn
        Kc = fzero(MsFun, ctrl(1)) ;            % Finding Kc
        ctrl = [Kc ID] ;
        try
            tic ;
        [ctrl iae_y optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
        catch
            fprintf('%s failed. Fminsearch results yields.\n',...
                    upper(solver))
        end
    end
    Msc  = Ms(ctrl) ;  % Calc. Ms for confirmation
    Ciaey = ctrl ;
    yOpt  = optFlag ;
    fprintf('%0.2f\t%.2f\t%.2f\t%.3f\t%.3f%d\t%3.2f\n',...
            iae_y,ctrl,Ms_s,Msc,optFlag,sTime)

    % Optimizing for IAE0_d
    init          = initValues(modelId) ; % Extracting initial values
    ctrl          = init(2,:) ;           % IAE0_d starting point
    parm.Jweights = [0 1] ;
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm) ;
    try
        tic ;
        [ctrl iae_d optFlag] = fmincon(Jfun, ctrl, [], [], [],...
                                    [], lb, [], cFun, opt) ;
        sTime = toc ;
    catch
        Jfun  = @(ID)noGradient([ctrl(1) ID], parm) ; % fsearch help function
        tic ;
        [ID iae_d optFlag] = fminsearch(Jfun, ctrl(2:end), opt) ;
        sTime = toc ;               % Ending CPU time measurement
        MsFun = @(Kc) parm.Ms_s - Ms([Kc ID]) ;   % fzero fcn
        Kc = fzero(MsFun, ctrl(1)) ;            % Finding Kc
        ctrl = [Kc ID] ;
    end
    Msc  = Ms(ctrl) ;  % Calc. Ms for confirmation
    Ciaed = ctrl ;
    dOpt  = optFlag ;
    fprintf('%0.2f\t%.2f\t%.2f\t%.3f\t%.3f%d\t%3.2f\n',...
            iae_d,ctrl,Ms_s,Msc,optFlag,sTime)
    
    res.iaeCtrl = [Ciaey; Ciaed] ;     % Saving controller tuning
    res.iae0    = [iae_y iae_d] ;            % Saving iae weights
    res.iaeFlags= [yOpt dOpt] ;                 % Saving optFlags
else
    %% Extracting data fra PID optimization
    data = load([main,'/results/pid/chriss/','POpid_',...
           num2str(modelName{modelId}),'.mat']) ;
    iae_y = data.res.POpid.weighting.IAE0_y ;
    iae_d = data.res.POpid.weighting.IAE0_d ;

        fprintf('\nIAE weights loaded.\n')
        fprintf('iae_y = %0.2f\n', iae_y)
        fprintf('iae_d = %0.2f\n', iae_d)
end

%% Pareto optimization
% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [iae_y iae_d]' ;                % IAE weights

% Generating pareto curve
i = 1 ;                                       % Iteration counter
ctrl    = init(3,:) ;   % Setting initial values for pareto curve
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Ms_clcs = zeros(length(Ms_span),1) ;       % Calculated Ms values 
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;   % IAE values per iteration
minTune = zeros(length(Ms_span),length(ctrl)) ;     % Opt. tuning
initial = zeros(length(Ms_span),length(ctrl)) ;    % Init. values
% ctrls   = zpk(zeros(length(Ms_span),1)) ;   % TFs for controllers
% ctrlRoot{length(Ms_span)} = [] ;  % Roots of produced controllers
    fprintf('\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J\tminTune\tMs_s\tMs\tflag\titer.\t')
    fprintf('time[s] \t timeleft[min] \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')


exitFlag = true ;
while exitFlag 
for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun      = @(ctrl)costFun(ctrl, parm) ;    % Updating cost
    cFun      = @(ctrl)conFun(ctrl,parm);% Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    runOpt = true ;
    while runOpt
        while sNum < 3
            try
                tic ;
                [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],...
                                           [],lb,[],cFun,opt); 
                             % Optimizing from last optimal point
                sTime(i) = toc ;    % Ending CPU time measurement
                if optFlag < sFlags{sNum}
                    fprintf('%s algorithm failed. ', ...
                            solvers{sNum}) 
                    sNum = sNum + 1 ;
                else
                    runOpt = false ;  % Opt success. Ending while
                    break
                end
            catch
                fprintf('%s algorithm failed. ', solvers{sNum})
                sNum = sNum + 1 ;
            end
        end
        if sNum == 3
            fprintf('Using %s:\n', solvers{sNum})
            if i == 1; ctrl = init(3,:) ;  % Reset initial values
            else; ctrl = minTune(i-1,:); end; % Last working ctrl
            Kc = ctrl(1) ; 
            ID = ctrl(2:end) ;
            Jfun = @(ID)noGrad2([Kc ID], parm) ;
            try
                tic ;
                [ID J optFlag] = ...
                                fminsearch(Jfun, ID, searchOpt) ;
                sTime(i) = toc ;
            catch
                fprintf('fminsearch failed.\nGiving up.\n')
                fprintf('Error message:\n'); disp(lasterr)
                runOpt = false ;
                exitFlag = false ;
                break
            end
            MsFun = @(Kc) Ms_s - Ms2(...
                    smithController([Kc ID])) ;
            Kc = fzero(MsFun, Kc) ;
            ctrl = [Kc ID] ;
            Jfun = @(ctrl)smithCost(ctrl, parm) ; % Updating cost
            cFun = @(ctrl)smithCon(ctrl,parm);% Updating constraints
            try
                fprintf('Second run sqp start... ')
                [ctrl J optFlag] = fmincon(Jfun, ctrl, [], ...
                                            [], [], [], lb,...
                                            [], cFun, opt);
                fprintf('Success!\n')
            catch
                fprintf('Second run sqp failed\n')
                ctrl = [Kc ID];
            end
            C = smithController(ctrl) ;
            MsCalc = MsSmith(C) ;
            if (MsCalc > Ms_s + 1e-4)||(MsCalc < Ms_s - 1e-4)
                fprintf('Ms values differ at control calculation. ')
                fprintf('Ms_s = %0.4f, Ms = \n',Ms_s, MsCalc)
            end
            runOpt = false ; % Opt success. Ending while.
        end
    end
    twice = 2 ;
    if exitFlag == false; break; end;
    C            = smithController(ctrl) ;% Generating controller
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Ms_clcs(i)   = MsSmith(C) ;            % Calculating Ms value
    Mt_span(i)   = MtSmith(C) ;          % Corresponding Mt-value
    % ctrls{i}     = C.tf ;                 % Storing controller TF
    % ctrlRoot{i}  = zpkdata(C.tf) ;             % Extracting roots
    IAEtune(i,:) = iae(C, parm) ;            % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft     = sum(sTime)/i*(length(Ms_span)-i)/60 ; 
                 % Estimation of time left for computations [min]
    fprintf('%.2f\t%.2f\t%.2f\t%.3f\t%.3f\t%d\t%d\t%0.2f\t\t%0.2f',...
        J,ctrl,Ms_s,Ms_clcs(i), optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
    sNum = sNum_s ;
end % Ending for-loop
exitFlag = false ;
end % Ending while-loop

calcTime = cputime - startTime ;  % End of total computation time

%% Ploting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(3) = plot(Msc,Jtune) ;            % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4',   ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8',    ...
            'Case 9', 'Case 10', 'Case 11', 'Case 12', ...
            'Case 13'} ;
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results
res.case = num2str(modelName{modelId});
res.G          = G ;
res.ctrlType   = ctrlType{cType} ;
res.ctrlr      = ctrlrs{ctrlr} ;
res.cParm      = cParm{cNum} ;
res.solver     = solver ;
res.IAEweights = parm.IAEweights ;

field = strcat(upper(ctrlType{cType},ctrlrs{ctrlr})) ;
res = setfield(res, field, Ms_span') ;
res = setfield(res, field, Msc) ;
res = setfield(res, field, Mt_span) ;
res = setfield(res, field, minTune) ;
res = setfield(res, field, Jtune) ;
res = setfield(res, field, flags) ;
res = setfield(res, field, inital) ;
res = setfield(res, field, calcTime) ;
res = setfield(res, field, IAEtune(:,1)) ;
res = setfield(res, field, IAEtune(:,2)) ;


diary([main, '/diary/', num2str(ctrlType{cType}), ...
      '/', num2str(ctrlrs{ctrlr}),'/', ...
      '/', num2str(cParm{cNum}),'/', ...
      num2str(modelName{modelId}),'.txt'])


% Saving struct and plot
save([main, '/results/', num2str(ctrlType{cType}), ...
      '/', num2str(ctrlrs{ctrlr}),'/', ...
      '/', num2str(cParm{cNum}),'/', ...
      num2str(modelName{modelId}),'.mat'])
saveas(h(3),[main, '/figures/', num2str(ctrlType{cType}), ...
      '/', num2str(ctrlrs{ctrlr}),'/', ...
      '/', num2str(cParm{cNum}),'/', ...
      num2str(modelName{modelId}),'.fig'])

fprintf('Calculation finished after %0.2f minutes', calcTime/60)
fprintf(' (%0.2f seconds). \n\n', calcTime)
end                                     % Ending modelId for-loop
runTime = cputime - runTime;
fprintf('Total computation time: %2.2f minutes', runTime/60)
fprintf(' (%2.2f seconds). \n\n', runTime)
diary off
matlabpool close
restoredefaultpath 