%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum ctrlr

startTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10',...
             'case11', 'case12', 'case13'} ;
ctrlType  = {'Chriss controller', 'Parallel', ...
             'Alternative parallel', 'Cascade'} ;

for modelId = 5:13 ;                                  % Model ID
parm.modelId = modelId ;
ctrlr = 'pid';
cNum    = 1 ;                                 % Controller option
sNum    = 2 ;                                     % Solver option
solvers = {'active-set', 'SQP'} ;   % Solvers
sFlags  = {5,1} ;                % Solver flag limits for success
solver  = solvers{sNum} ;                       % Choosing solver
sFlag   = sFlags{sNum}  ;                   % Choosing flag limit

simTime = 100 ;                                 % Simulation time
MsRes   = 0.025 ;                         % Ms PO plot resolution
G       = model(modelId) ;  % Generating system transfer function
parm.cNum = cNum ;
parm.G = G ;

%% Start diary
diary([main,'/diary/pid/','diary_POpid_',...
       num2str(modelName{modelId}),'.txt'])

%% Parameters for finding IAE weights
parm.Ms_s = 1.59 ;                                  % Ms setpoint
parm.simTime = simTime ;               % Response simulation time
parm.IAEweights = [1 1]' ;   % Setting IAE cost function weigting

init = initValues(modelId) ;             % Initial control matrix
lb = [0 0 0]' ;                              % Lower tuning bound


% Optimizer options
opt = optimoptions('fmincon','algorithm',solver,'UseParallel',...
                   'always','Display','off','TolCon',...
                1e-4,'PlotFcns',{@optimplotx, ...
                @optimplotfval, @optimplotfunccount}); 
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','MaxFunEvals', 80, 'PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Controller: %s\n', ctrlType{cNum})
    fprintf('Solver: %s\n\n', opt.Algorithm)
    
if cNum == 1
    fprintf('Finding Optimal IAE Weights \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning\n')
    fprintf('J     \t Kc \t I \t D \t Ms \t flag \t CPU time\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    %% Optimizing for IAE0_y
    parm.Jweights   = [1 0]  ;    % Setting cost function weights
    ctrl = init(1,:) ;        % Setting initial values for IAE0_y
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm);
    tic ;                         % Starting CPU time measurement
    [ctrl iae_y optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                            [],cFun,opt);
    sTime = toc ;                   % Ending CPU time measurement
    res.POpid.weighting.ctrl_y = ctrl ;% Saving controller settings
    res.POpid.weighting.iae_y = iae_y ;% Saving IAE input weight
    res.POpid.weighting.flag_y = optFlag ;     % Saving exit flag

        fprintf('%0.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%d\t%0.2f\n',...
            iae_y,ctrl,parm.Ms_s,optFlag,sTime)

    % Optimizing for iae_d
    parm.Jweights = [0 1]  ;
    ctrl = init(2,:) ;        % Setting initial values for iae_d
    Jfun   = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl,parm);
    tic ;                         % Starting CPU time measurement
    [ctrl iae_d optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                            [],cFun,opt);
    sTime = toc ;                   % Ending CPU time measurement
    res.POpid.weighting.ctrl_d = ctrl ;% Saving controller settings
    res.POpid.weighting.iae_d = iae_d ;% Saving IAE disturbance weight
    res.POpid.weighting.flag_d = optFlag ;     % Saving exit flag

        fprintf('%0.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%d\t%0.2f\n',...
            iae_d,ctrl,parm.Ms_s,optFlag,sTime)
else
    data = load([main,'/results/pid/chriss/','POpid_',...
           num2str(modelName{modelId}),'.mat']) ;
    iae_y = data.res.POpid.weighting.iae_y ;
    iae_d = data.res.POpid.weighting.iae_d ;

    fprintf('IAE weights loaded.\n')
    fprintf('iae_y = %0.2f\n', iae_y)
    fprintf('iae_d = %0.2f\n', iae_d)
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning  \t   \t      \t \n')
    fprintf('J     \t Kc \t I \t D \t Ms \t flag \t CPU time\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
end
%% Pareto optimization
% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [iae_y iae_d]' ;                % IAE weights
ctrl = init(3,:) ;      % Setting initial values for pareto curve

% Generating pareto curve
i = 1 ;                                       % Iteration counter
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Msc     = zeros(length(Ms_span),1) ;
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;           % Init. IAE vector
minTune = zeros(length(Ms_span),3) ;        % Init. tuning vector
initial = zeros(length(Ms_span),3) ;    % Initial values for each iteration
ctrls   = zpk(zeros(length(Ms_span),1)) ; % Transfer functions for controllers
ctrlRoot{length(Ms_span)} = [] ; % Roots of produced controllers

    fprintf('\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J \t minTune \t \t Ms_s\tMs\tflag\titer.\t')
    fprintf('time[s] \t timeleft[min] \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun   = @(ctrl)costFun(ctrl, parm) ;         % Updating cost
    cFun = @(ctrl)conFun(ctrl,parm);       % Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    try
        tic ;                     % Starting CPU time measurement
        [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                                   [],cFun,opt);% Optimizing from 
                                             % last optimal point
        sTime(i) = toc ;            % Ending CPU time measurement
    catch
        optFlag = 0 ;        % Set optFlag to zero upon opt error
    end
    if optFlag < sFlag         % Gradient free opt upon exit flag
        fprintf('%s algorithm failed. Using fminsearch:\n', ...
                    opt.Algorithm)
        if i == 1; ctrl = init(3,:) ;  % Resetting initial values
        else; ctrl = minTune(i-1,:); end ;    % Last working ctrl
        Kc = ctrl(1)            % Setting Kc for noGradient
        ID = ctrl(2:end)   % Setting I and D for noGradient
        Jfun  = @(ID)noGradient([Kc ID],parm) ;% fsearch help function
        try
            tic ;                     % Starting CPU time measurement
            [ID J optFlag] = ...
                           fminsearch(Jfun,ID, searchOpt) ;
            sTime(i) = toc ;            % Ending CPU time measurement
        catch
            fprintf('fminsearch failed.\n')
            fprintf('Error message:\n'); disp(lasterr)
            runOpt = false ;
            exitFlag = false ;
            break
        end
        MsFun = @(Kc) Ms_s - Ms([Kc ID]) ;        % help function
        Kc = fzero(MsFun, Kc) ;         % Finding controller gain
        ctrl = [Kc ID] ;           % Generating controller vector
        Jfun = @(ctrl)costFun(ctrl, parm) ;
        cFun = @(ctrl)conFun(ctrl, parm) ;
        try
            fprintf('Second rund %s start\n', ...
                    upper(solvers{sNum})) ;
            [ctrl J optFlag] = fmincon(Jfun, ctrl, [], ...
                                       [], [], [], lb, ...
                                       [], cFun, opt) ;
        catch
            fprintf('Second run %s failed.\n', ...
                    upper(solvers{sNum})) ;
            ctrl = [Kc ID]
        end
    end
    C = controller(ctrl) ;
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Mt_span(i)   = Mt(ctrl) ;% Calculating corresponding Mt-value
    Msc(i)       = Ms(ctrl) ;
    ctrls(i)     = C.tf ;                 % Storing controller TF
    ctrlRoot{i}  = zpkdata(C.tf) ;             % Extracting roots
    IAEtune(i,:) = iae(C, parm) ;            % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]
    fprintf('%.2f\t%.2f\t%.2f\t%.2f\t%.3f\t%.3f\t%d\t%d\t%0.2f\t\t%0.2f',...
        J,ctrl,Ms_s,Msc(i),optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

calcTime = cputime - startTime ;  % End of total computation time

%% Ploting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(3) = plot(Ms_span,Jtune);         % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2',  'Case 3',  'Case 4', ...
             'Case 5', 'Case 6',  'Case 7',  'Case 8', ...
             'Case 9', 'Case 10', 'Case 11', 'Case 12', ...
             'Case 13'} ;
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results
res.case = num2str(modelName{modelId});
res.ctrlType       = ctrlType{cNum} ;
res.G              = G ;
res.solver         = solver ;
res.IAEweights     = parm.IAEweights ;
res.POpid.Ms       = Ms_span ;
res.POpid.Mt       = Mt_span ;
res.POpid.tuning   = minTune ;
res.POpid.Jtune    = Jtune ;
res.POpid.flags    = flags ;
res.POpid.ctrls    = ctrls ; 
res.POpid.initial  = initial ;
res.POpid.calcTime = calcTime ;
res.POpid.Jy       = IAEtune(:,1) ;
res.POpid.Jd       = IAEtune(:,2) ;
res.POpid.ctrls    = ctrls ; 
res.POpid.roots    = ctrlRoot ;

% Saving struct and plot
save([main,'/results/pid/chriss/','POpid_',...
      num2str(modelName{modelId}),'.mat'],'res')  
saveas(h(3),[main,'/figures/pid/chriss/', 'POpidCurve_', ...
      num2str(modelName{modelId}),'.fig'])

fprintf('Calculation finished after %0.2f minutes', calcTime/60)
fprintf(' (%0.2f seconds). \n\n', calcTime)

end                                     % Ending modelId for-loop
runTime = cputime - runTime;
fprintf('Total computation time: %2.2f minutes', runTime/60)
fprintf(' (%2.2f seconds). \n\n', runTime)
diary off
restoredefaultpath 