%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

startTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9'} ;
ctrlType  = {'Chriss controller', 'Parallel', ...
             'Alternative parallel', 'Cascade'} ;

for modelId = 1:9 ;                                    % Model ID
cNum    = 4 ;                                 % Controller option
sNum    = 1 ;                                     % Solver option
solvers = {'active-set', 'sqp'} ;   % Solvers
sFlags  = {5,1} ;                % Solver flag limits for success
solver  = solvers{sNum} ;                       % Choosing solver
sFlag   = sFlags{sNum}  ;                   % Choosing flag limit

parm.simTime = 100 ;                            % Simulation time
MsRes   = 0.025 ;                         % Ms PO plot resolution
G       = model(modelId) ;  % Generating system transfer function

%% Start diary
diary([main,'/diary/cascade_pid/','diary_cPID_',...
       num2str(modelName{modelId}),'.txt'])

% Optimizer options
opt = optimset('algorithm',solver,'Display','off','TolCon',...
        1e-4,'PlotFcns',...
        {@optimplotx, @optimplotfval, @optimplotfunccount}); 
    % 'active-set', 'trust-region-reflective', 'interior-point',
    % 'interior-point-convex', 'levenberg-marquardt', 
    % 'trust-region-dogleg', 'lm-line-search', or 'sqp'.
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Controller: %s\n', ctrlType{cNum})
    fprintf('Solver: %s\n\n', opt.Algorithm)

%% Extracting data fra PID optimization
data = load([main,'/results/pid/chriss/','POpid_',...
            num2str(modelName{modelId}),'.mat']) ;

IAE0_y = data.res.POpid.weighting.IAE0_y ;
IAE0_d = data.res.POpid.weighting.IAE0_d ;

    fprintf('IAE weights loaded.\n')
    fprintf('IAE0_y = %0.2f\n', IAE0_y)
    fprintf('IAE0_d = %0.2f\n', IAE0_d)

%% Pareto optimization
lb = [0 0 0]' ;                              % Lower tuning bound
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights
ctrl = initValues(modelId) ;             % Setting initial values

% Generating pareto curve
i = 1 ;                                       % Iteration counter
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;           % Init. IAE vector
minTune = zeros(length(Ms_span),3) ;        % Init. tuning vector
initial = zeros(length(Ms_span),3) ;    % Initial values at each iteration
ctrls   = zpk(zeros(length(Ms_span),1)) ; % Transfer functions for controllers
ctrlRoot{length(Ms_span)} = [] ; % Roots of produced controllers

    fprintf('\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J \t K \t I \t D \t Ms \t flag \t iter. \t time[s] \t timeleft[min] \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun = @(ctrl)costFun(ctrl, parm) ;           % Updating cost
    cFun = @(ctrl)conFun(ctrl,parm) ;      % Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    try
        tic ;                     % Starting CPU time measurement
        [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                           [],cFun,opt);   % Optimizing from last 
                                                  % optimal point
        sTime(i) = toc ;            % Ending CPU time measurement
    catch
        optFlag = 0 ;
    end
    if optFlag < 1    % Gradient free optimization upon exit flag
        fprintf('%s algorithm failed. Using fminsearch:\n', ...
                    opt.Algorithm)
        if i == 1
            ctrl = initValues(modelId) ;  % Resetting init values
        else
            ctrl = minTune(i-1,:) ; % Resetting last working ctrl
        end
        parm.Kc = ctrl(1) ;           % Setting Kc for noGradient
        parm.ID = ctrl(2:end) ;       % Setting ID for noGradient
        Jfun  = @(ID)noGradient(ID,parm) ;% fsearch help function
        tic ;                     % Starting CPU time measurement
        [parm.ID J optFlag] = ...
                       fminsearch(Jfun,parm.ID, searchOpt) ;
        sTime(i) = toc ;            % Ending CPU time measurement
        MsFun = @(Kc) Ms_s - Ms([Kc parm.ID]) ;% fzero help function
        parm.Kc = fzero(MsFun,parm.Kc) ;% Finding controller gain
        ctrl = [parm.Kc parm.ID] ; % Generating controller vector
        MsCalc  = Ms(ctrl) ;    % Calculating Ms for confirmation
        if (MsCalc > Ms_s+1e-4)||(MsCalc < Ms_s-1e-4)
            fprintf('Ms value differs at control calculation. ')
            fprintf('MsCalc = %0.4f\n',MsCalc)
        end
    end
    C = controller(ctrl) ;          % Producing controller struct
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Mt_span(i) = Mt(ctrl) ;  % Calculating corresponding Mt-value
    ctrls(i) = C.tf ;                     % Storing controller TF
    ctrlRoot{i} = zpkdata(C.tf) ;              % Extracting roots
    IAEtune(i,:) = iae(ctrl, parm) ;         % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]
    fprintf('%.2f \t %.2f \t %.2f \t %.2f \t %.3f \t %d \t %d \t %0.2f \t\t %0.2f',...
        J,ctrl,Ms_s,optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

calcTime = cputime - startTime ;  % End of total computation time

%% Plotting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(1) = plot(Ms_span,Jtune);         % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8', 'Case 9'};
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results
res.case = num2str(modelName{modelId});
res.ctrlType        = ctrlType{cNum} ;
res.G               = G ;
res.ctrlType        = ctrlType{cNum} ;
res.solver          = solver ;
res.IAEweights      = parm.IAEweights ;
res.POcasc.Ms       = Ms_span ;
res.POcasc.Mt       = Mt_span ;
res.POcasc.tuning   = minTune ;
res.POcasc.Jtune    = Jtune ;
res.POcasc.flags    = flags ;
res.POcasc.ctrls    = ctrls ; 
res.POcasc.initial  = initial ;
res.POcasc.calcTime = calcTime ;
res.POcasc.Jy       = IAEtune(:,1) ;
res.POcasc.Jd       = IAEtune(:,2) ;
res.POcasc.ctrls    = ctrls ; 
res.POcasc.roots    = ctrlRoot ;

save([main,'/results/pid/cascade/','POcasc_',...
      num2str(modelName{modelId}),'.mat'],'res')   % Saving struct
saveas(h(1),[main,'/figures/pid/cascade','POcascCurve_',...
      num2str(modelName{modelId}),'.fig'])      % Saving figures

fprintf('Calculation finished after %0.2f minutes', calcTime/60)
fprintf(' (%0.2f seconds). \n\n', calcTime)

end                                             % Ending for-loop
diary off
restoredefaultpath