%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    parm = 
%               .modelId                          Model ID number
%               .Jweights                   Cost function weights
%               .IAEweights                  IAE function weights
%               .Ms_s                             Setpoint for Ms
%               .simTime                     Simulation time span
%               .cTime                    Computation time vector

function parm = findOpt(parm)
    global G

    inits = initValues(parm.modelId) ;  % Retrieving init. values
    if parm.Jweights == [1 0]                    % Finding IAE0_y
        ctrl0 = inits(1,:) ;
    elseif parm.Jweights == [0 1]                % Finding IAE0_d
        ctrl0 = inits(2,:) ;
    else                           % Finding pareto optimal curve
        ctrl0 = inits(3,:) ;
    end

    i = 1 ;                                   % Iteration counter
    Ms_span = 1.25:MsRes:3 ;                      % Values for Ms
    sTime   = zeros(length(Ms_span),1) ; % Init. time consumption
    Jtune   = zeros(length(Ms_span),1) ;      % Init. cost vector
    flags   = zeros(length(Ms_span),1) ;% Init. exit flags vector
    IAEtune = zeros(length(Ms_span),2) ;       % Init. IAE vector
    minTune = zeros(length(Ms_span),3) ;    % Init. tuning vector

    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm);
    tic ;                             % Starting CPU time measurement
    [ctrl IAE0_y optFlag] = fmincon(Jfun,ctrl0,[],[],[],[],lb, ...
                        [],cFun,opt);
    parm.cTime = toc ;                   % Ending CPU time measurement

    if optFlag < 1    % Gradient free optimization upon exit flag
        fprintf('SQP-solver failed. Using fminsearch.\n')
        if i == 1
            ctrl = ctrl0 ;             % Resetting initial values
        else
            ctrl = minTune(i-1,:) ; % Resetting last working ctrl
        end
        parm.Kc = ctrl(1) ;           % Setting Kc for noGradient
        parm.ID = ctrl(2:end) ;  % Setting I and D for noGradient
        Jfun  = @(ID)noGradient(ID,parm) ;% fsearch help function
        tic ;                     % Starting CPU time measurement
        [parm.ID J optFlag] = ...
                       fminsearch(Jfun,parm.ID, searchOpt) ;
        cTime(i) = toc ;            % Ending CPU time measurement
        MsFun = @(Kc) Ms_s - Ms([Kc parm.ID]) ;% fzero help function
        parm.Kc = fzero(MsFun,parm.Kc) ;% Finding controller gain
        ctrl = [parm.Kc parm.ID] ; % Generating controller vector
        MsCalc  = Ms(ctrl) ;    % Calculating Ms for confirmation
        if MsCalc ~= abs(Ms_s+1e-4)
            fprintf('\n !!! Ms value differs at control calculation. MsCalc = %0.2f\n\n',MsCalc)
        end
    end
    res.PO.setpoints.ctrl_y = ctrl ;     % Saving controller settings
    res.PO.setpoints.IAE0_y = IAE0_y ;      % Saving IAE input weight
    res.PO.setpoints.flag_y = optFlag ;            % Saving exit flag



end
