%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lødemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013

% Optimal [Kc I D] for IAE tuning: 3.6689 0.9100 0.0789


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
pid    = fullfile(main,'PID') ;
cascade= fullfile(main,'cascade') ;
addpath(shared)
addpath(pid)
addpath(cascade)

%% Globals
global G cNum

startTime = cputime ;

%% Process and controller parameters
for modelId = 10:10 ;

MsRes   = 0.025 ;                        % Resolution of Ms space
G       = model(modelId) ;  % Generating system transfer function
res.G   = G ;                   % Saving system transfer function
cNum    = 1 ;                                 % Controller option
modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9', 'case10',...
             'case11', 'case12', 'case13'} ;
ctrlType = {'Chriss', 'Parallel', 'Alt.Paralell', 'Cascade'};
parm.simTime = 100 ;                   % Response simulation time

    fprintf('Starting main_noGrad.m \n')
    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)

% Solver options
opt = optimset('TolFun',1e-3,'TolX',1e-4,'Display','off', ...
               'PlotFcns',{@optimplotx, @optimplotfval, ... 
               @optimplotfunccount}) ;

if cNum == 1
    %% Parameters for finding IAE weights
    parm.Ms_s       = 1.59 ;                        % Ms setpoint
    parm.IAEweights = [1 1]' ;% Setting IAE cost function weights

    fprintf('Finding Optimal IAE Weights \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning  \t   \t      \t \n')
    fprintf('J     \t Kc \t I \t D \t Ms \t flag \t CPU time \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    %% IAE Weights
    % Optimizing for IAE0_y
    init          = initValues(modelId) ; % Extracting initial values
    ctrl          = init(1,:) ;           % IAE0_y starting point
    parm.Kc       = ctrl(1) ;           % Initial controller gain
    parm.ID       = ctrl(2:end) ;         % Initial [tau_i tau_d]
    parm.Jweights = [1 0]  ;      % Setting cost function weights
    Jfun  = @(ID)noGradient(ID, parm) ;   % fsearch help function
    tic ;                         % Starting CPU time measurement
    [ID IAE0_y optFlag] = fminsearch(Jfun, parm.ID, opt) ;
    sTime = toc ;                   % Ending CPU time measurement
    MsFun = @(Kc) parm.Ms_s - Ms([Kc ID]) ; % fzero help function
    parm.Kc = fzero(MsFun, parm.Kc) ;   % Finding controller gain
    parm.ID = ID ;                           % Updating ID values
    MsCalc  = Ms([parm.Kc ID]) ;% Calculating Ms for confirmation
    res.iaey = ctrl ;

        fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %.2f \t %d \t %0.2f \n',...
            IAE0_y,[parm.Kc parm.ID],MsCalc,optFlag,sTime)

    % Optimizing for IAE0_d
    init          = initValues(modelId) ; % Extracting initial values
    ctrl          = init(2,:) ;           % IAE0_d starting point
    parm.Kc       = ctrl(1) ;           % Initial controller gain
    parm.ID       = ctrl(2:end) ;         % Initial [tau_i tau_d]
    parm.Jweights = [0 1]  ;
    Jfun  = @(ID)noGradient(ID, parm) ;   % fsearch help function
    tic ;                         % Starting CPU time measurement
    [ID IAE0_d optFlag] = fminsearch(Jfun, parm.ID, opt) ;
    sTime = toc ;                   % Ending CPU time measurement
    MsFun = @(Kc) parm.Ms_s - Ms([Kc ID]) ; % fzero help function
    parm.Kc = fzero(MsFun, parm.Kc) ;   % Finding controller gain
    parm.ID = ID ;                           % Updating ID values
    MsCalc  = Ms([parm.Kc ID]) ;% Calculating Ms for confirmation
    res.iaed = ctrl ;

        fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %.2f \t %d \t %0.2f \n',...
            IAE0_d,[parm.Kc parm.ID],MsCalc,optFlag,sTime)
else
    data = load([main,'/results/pid/chriss/','POpid_',...
            num2str(modelName{modelId}),'.mat']) ;
    try
        IAE0_y = data.res.PO.weighting.IAE0_y ;
    catch
        IAE0_y = data.res.POpid.weighting.IAE0_y ;
    end
    try
        IAE0_d = data.res.PO.weighting.IAE0_d ;
    catch
        IAE0_d = data.res.POpid.weighting.IAE0_d ;
    end
    fprintf('IAE weights loaded.\n')
    fprintf('IAE0_y = %0.2f\n', IAE0_y)
    fprintf('IAE0_d = %0.2f\n', IAE0_d)
end


%% Setting weights for pareto curve generation
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights
init = initValues(modelId) ;             % Initial control matrix
if cNum == 4
    parm.Kc = init(1) ;
    parm.ID = init(2:end) ;     % Starting point for pareto curve
else
    parm.Kc = init(3,1) ;
    parm.Id = init(3,2:end) ;
end

% Generating pareto curve
i = 1 ;                                       % Iteration counter
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
IAEtune = zeros(length(Ms_span),2) ;           % Init. IAE vector
minTune = zeros(length(Ms_span),3) ;        % Init. tuning vector
initial = zeros(length(Ms_span),3) ;    % Initial values for each iteration
ctrls   = zpk(zeros(length(Ms_span),1)) ; % Transfer functions for controllers
ctrlRoot{length(Ms_span)} = [] ; % Roots of produced controllers

    fprintf('\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J \t minTune \t \t Ms \t flag \t it.left  \t CPU time \t Est.timeleft\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun  = @(ID)noGradient(ID, parm) ;   % fsearch help function
    tic ;                         % Starting CPU time measurement
    [ID J optFlag] = fminsearch(Jfun, parm.ID, opt) ;
    sTime(i) = toc ;                % Ending CPU time measurement
    MsFun = @(Kc) parm.Ms_s - Ms([Kc ID]) ; % fzero help function
    parm.Kc = fzero(MsFun, parm.Kc) ;   % Finding controller gain
    parm.ID = ID ;                           % Updating ID values
    ctrl = [parm.Kc ID] ;                % Generating ctrl vector
    MsCalc  = Ms(ctrl) ;        % Calculating Ms for confirmation
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    Mt_span(i) = Mt(ctrl) ;  % Calculating corresponding Mt-value
    C = controller(ctrl) ;          % Producing controller struct
    ctrls(i) = C.tf ;                     % Storing controller TF
    ctrlRoot{i} = zpkdata(C.tf) ;              % Extracting roots
    IAEtune(i,:) = iae(ctrl, parm) ;         % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]
    fprintf('%.2f \t %.2f \t %.2f \t %.2f \t %.3f \t %d \t %d \t %0.2f \t\t %0.2f',...
        J,ctrl,Ms_s,optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

calcTime = cputime - startTime ;  % End of total computation time

%% Plotting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h = plot(Ms_span,Jtune);            % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$','interpreter','latex','FontSize',14)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',14)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8', ...
            'Case 9', 'Case 10', 'Case 11', 'Case 12', ...
            'Case 13'} ;
title(titleName{modelId},'interpreter','latex','FontSize',14)

%% Storing results
res.case = num2str(modelName{modelId});
res.ctrlType = ctrlType(cNum) ;
res.note = 'Gradient free' ;
res.POng.Ms     = Ms_span ;
res.POng.Mt     = Mt_span ;
res.POng.tuning = minTune ;
res.POng.Jtune  = Jtune ;
res.POng.flags  = flags ;
res.POng.ctrls  = ctrls ; 
res.POng.initial = initial ;
res.POng.calcTime = calcTime ;
res.POng.Jy = IAEtune(:,1) ;
res.POng.Jd = IAEtune(:,2) ;
res.POng.ctrls = ctrls ; 
res.POng.roots = ctrlRoot ;


% Saving struct and plot
save([main,'/results/nograd/',ctrlType{cNum},...
     num2str(modelName{modelId}),'.mat'],'res')
saveas(h,['figures/nograd/',ctrlType{cNum},...
       num2str(modelName{modelId}),'.fig'])

end                                             % Ending for-loop
diary off
restoredefaultpath