%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

runTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
calcWeights = 0 ; % Calculate IAE weights? Y(1), N(0)
modelName = {'case10', 'case11', 'case12', 'case13'} ;
ctrlType  = {'pi', 'pid_par', 'pid_casc', 'null'} ;

for modelId = 3:4 ;                                    % Model ID
startTime = cputime ;

cNum    = 1 ;                                 % Controller option
sNum    = 1 ;                                     % Solver option
solvers = {'active-set', 'sqp'} ;   % Solvers
sFlags  = {5,1} ;                % Solver flag limits for success
solver  = solvers{sNum} ;                       % Choosing solver
sFlag   = sFlags{sNum}  ;                   % Choosing flag limit

simTime = 100 ;                                 % Simulation time
MsRes   = 0.025 ;                         % Ms PO plot resolution
G       = smithModel(modelId) ;           % Generating process TF

%% Start diary
diary([main,'/diary/smith/','POsmithPI_',...
       num2str(modelName{modelId}),'.txt'])

%% Parameters for finding IAE weights
parm.Ms_s = 1.59 ;                                  % Ms setpoint
parm.simTime = simTime ;               % Response simulation time
init = smithInit(modelId) ;   % Initial control matrix
lb = zeros(size(init,2),1) ;                 % Lower tuning bound

% Optimizer options
opt       = optimset('algorithm',solver,'Display','off',...
            'TolCon',1e-4,'PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ; 
searchOpt = optimset('TolFun',1e-3,'TolX',1e-4,...
            'Display','off','PlotFcns',...
            {@optimplotx, @optimplotfval, @optimplotfunccount}) ;

    fprintf('SMITH PREDICTOR \n')
    fprintf('Main Directory: %s\n', main)
    fprintf('Case: %g\n',modelId)
    fprintf('Controller: %s\n', ctrlType{cNum})
    fprintf('Solver: %s\n\n', opt.Algorithm)
    
if calcWeights
    fprintf('Finding Optimal IAE Weights \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Cost\tTuning\n')
    fprintf('J\tKc\tI\tMs\tflag\tCPU time\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    %% Optimizing for IAE0_y
    parm.IAEweights = [1 1]' ;     % Setting IAE costFun weigting
    parm.Jweights   = [1 0]  ;          % Setting costFun weights
    ctrl = smithInit(1,:) ;         % Setting initial ctrl values
    Jfun = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl, parm);
    tic ;                         % Starting CPU time measurement
    [ctrl IAE0_y optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                            [],cFun,opt);
    sTime = toc ;                   % Ending CPU time measurement
    res.POpid.weighting.ctrl_y = ctrl ;        % Saving ctrl vec.
    res.POpid.weighting.IAE0_y = IAE0_y ;   % Saving IAE_y weight
    res.POpid.weighting.flag_y = optFlag ;     % Saving exit flag

        fprintf('%0.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%d\t%0.2f\n',...
                IAE0_y,ctrl,parm.Ms_s,optFlag,sTime)

    % Optimizing for IAE0_d
    parm.Jweights = [0 1]  ;
    ctrl = init(2,:) ;        % Setting initial values for IAE0_d
    Jfun   = @(ctrl)costFun(ctrl, parm) ;
    cFun = @(ctrl)conFun(ctrl,parm);
    tic ;                         % Starting CPU time measurement
    [ctrl IAE0_d optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                            [],cFun,opt);
    sTime = toc ;                   % Ending CPU time measurement
    res.POpid.weighting.ctrl_d = ctrl ;        % Saving ctrl vec.
    res.POpid.weighting.IAE0_d = IAE0_d ;   % Saving IAE_d weight
    res.POpid.weighting.flag_d = optFlag ;     % Saving exit flag

        fprintf('%0.2f\t%.2f\t%.2f\t%.2f\t%.2f\t%d\t%0.2f\n',...
                IAE0_d,ctrl,parm.Ms_s,optFlag,sTime)
else
    data = load([main,'/results/pi/','POpi_',...
           num2str(modelName{modelId}),'.mat']) ;
    IAE0_y = data.res.IAEweights(1) ;
    IAE0_d = data.res.IAEweights(2) ;

    fprintf('IAE weights loaded.\n')
    fprintf('IAE0_y = %0.2f\n', IAE0_y)
    fprintf('IAE0_d = %0.2f\n', IAE0_d)
end

%% Pareto optimization
% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights

% Generating pareto curve
i = 1 ;                                       % Iteration counter
ctrl    = init(3,:) ;   % Setting initial values for pareto curve
Ms_span = 1.25:MsRes:3 ;                          % Values for Ms
Mt_span = zeros(length(Ms_span),1) ;            % Init. Mt vector
sTime   = zeros(length(Ms_span),1) ;     % Init. time consumption
Jtune   = zeros(length(Ms_span),1) ;          % Init. cost vector
flags   = zeros(length(Ms_span),1) ;    % Init. exit flags vector
IAEtune = zeros(length(Ms_span),2) ;   % IAE values per iteration
minTune = zeros(length(Ms_span),length(ctrl)) ;     % Opt. tuning
initial = zeros(length(Ms_span),length(ctrl)) ;    % Init. values
% ctrls   = zpk(zeros(length(Ms_span),1)) ;   % TFs for controllers
% ctrlRoot{length(Ms_span)} = [] ;  % Roots of produced controllers
    fprintf('\n')
    fprintf('Generating the Pareto Optimal Curve\n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')
    fprintf('Number of iterations: %d\n', length(Ms_span))
    fprintf('J \t minTune \t \t Ms \t flag \t iter. \t')
    fprintf('time[s] \t timeleft[min] \n')
    fprintf('------------------------------------------------')
    fprintf('------------------------------------------------\n')

for Ms_s = Ms_span
    parm.Ms_s = Ms_s ;               % Updating Ms setpoint value
    Jfun      = @(ctrl)smithCost(ctrl, parm) ;    % Updating cost
    cFun      = @(ctrl)smithCon(ctrl,parm);% Updating constraints
    initial(i,:) = ctrl ;% Saving starting point for optimization
    % try
    %     tic ;                     % Starting CPU time measurement
    %     [ctrl J optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
    %                                [],cFun,opt);% Optimizing from 
    %                                          % last optimal point
    %     sTime(i) = toc ;            % Ending CPU time measurement
    % catch
    %     optFlag = 0 ;        % Set optFlag to zero upon opt error
    % end
    optFlag = 0 ;
    if optFlag < sFlag         % Gradient free opt upon exit flag
        fprintf('%s algorithm failed. Using fminsearch:\n', ...
                    opt.Algorithm)
        if i == 1
            ctrl = init(3,:) ;         % Resetting initial values
        else
            ctrl = minTune(i-1,:) ; % Resetting last working ctrl
        end
        parm.Kc = ctrl(1) ;           % Setting Kc for noGradient
        parm.ID = ctrl(2:end) ;        % I (and D) for noGradient
        Jfun  = @(ID)noGradient(ID,parm) ;% fsearch help function
        tic ;                     % Starting CPU time measurement
        [parm.ID J optFlag] = ...
                       fminsearch(Jfun,parm.ID, searchOpt) ;
        sTime(i) = toc ;            % Ending CPU time measurement
        MsFun = @(Kc) Ms_s - Ms([Kc parm.ID]) ;   % help function
        parm.Kc = fzero(MsFun,parm.Kc) ;% Finding controller gain
        ctrl = [parm.Kc parm.ID] ; % Generating controller vector
        MsCalc  = Ms(ctrl) ;    % Calculating Ms for confirmation
        if (MsCalc > Ms_s + 1e-4)||(MsCalc < Ms_s - 1e-4)
            fprintf('Ms value differs at control calculation. ')
            fprintf('MsCalc = %0.4f\n',MsCalc)
        end
    end
    C = smithController(ctrl) ;     % Producing controller struct
    minTune(i,:) = ctrl ;             % Storing controller tuning
    Mt_span(i)   = MtSmith(C) ;          % Corresponding Mt-value
    % ctrls{i}     = C.tf ;                 % Storing controller TF
    % ctrlRoot{i}  = zpkdata(C.tf) ;             % Extracting roots
    IAEtune(i,:) = iae(C, parm) ;            % Saving IAE weights
    Jtune(i,:)   = J    ;           % Storing cost function value
    flags(i,:)   = optFlag ;                  % Storing exit flag
    timeleft = sum(sTime)/i*(length(Ms_span)-i)/60 ; % Estimation
                            % of time left for computations [min]
    fprintf('%.2f\t%.2f\t%.2f\t%.3f\t%d\t%d\t%0.2f\t\t%0.2f',...
        J,ctrl,Ms_s,optFlag,length(Ms_span)-i,sTime(i),timeleft)
    fprintf('\n')
    i = i + 1 ;                            % Incrementing counter
end

calcTime = cputime - startTime ;  % End of total computation time

%% Ploting routine
figure(modelId)                 % Figure number from model number
clf                                                % Clear figure
h(3) = plot(Ms_span,Jtune);         % Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
            'Case 5', 'Case 6', 'Case 7', 'Case 8', 'Case 9'} ;
title(titleName{modelId},'interpreter','latex','FontSize',12)

%% Storing results
res.case = num2str(modelName{modelId});
res.ctrlType         = ctrlType{cNum} ;
res.G                = G ;
res.solver           = solver ;
res.IAEweights       = parm.IAEweights ;
res.smithPI.Ms       = Ms_span ;
res.smithPI.Mt       = Mt_span ;
res.smithPI.tuning   = minTune ;
res.smithPI.Jtune    = Jtune ;
res.smithPI.flags    = flags ;
res.smithPI.initial  = initial ;
res.smithPI.calcTime = calcTime ;
res.smithPI.Jy       = IAEtune(:,1) ;
res.smithPI.Jd       = IAEtune(:,2) ;
% res.smithPI.ctrls    = ctrls ; 
% res.smithPI.roots    = ctrlRoot ;

% Saving struct and plot
save([main,'/results/pi/smith/','smithPI_',...
      num2str(modelName{modelId}),'.mat'],'res')  
saveas(h(3),[main,'/figures/pi/smith/', 'smithPI_', ...
      num2str(modelName{modelId}),'.fig'])
fprintf('Calculation finished after %0.2f minutes', calcTime/60)
fprintf(' (%0.2f seconds). \n\n', calcTime)
end                                     % Ending modelId for-loop
runTime = cputime - runTime;
fprintf('Total computation time: %2.2f minutes', runTime/60)
fprintf(' (%2.2f seconds). \n\n', runTime)
diary off
restoredefaultpath 